unit demo_unit;

interface

uses graphics,types;

var GoDemoFlag : boolean;

procedure startdemo(nr : byte);
procedure createscanmap;
procedure destroyscanmap;

implementation

uses unit1;

type Tdemotext = record
                  x : smallInt;
                  y : smallInt;
                  height : byte;
                  s : string;
                 end;

const demotext : array[1..5] of TDemotext =
               ((x: -150;y:-300;height:40;s:'H(ro)Bot simulator'),
                (x: -150;y:-240;height:40;s:'DavData Software'),
                (x: -150;y:-180;height:30;s:'august 2023'),
                (x: -150;y:  40;height:30;s:'https://davdata.nl'),
                (x:    0;y: 300;height:30;s:' '));


var scanmap : Tbitmap;

procedure movetoXY(x2,y2 : smallInt);
const code2dir : array[1..$a] of byte = (0,4,$ff,6,7,5,$ff,2,1,3);
var x1,y1,dx,dy : smallInt;
    stepX,stepY : single;
    n,steps : word;
    pX,pY,newX,newY : smallInt;
    code,dir : byte;
begin
 GetPenPos(x1,y1);
 dx := x2-x1;
 dy := y2-y1;
 if (dx=0) and (dy=0) then exit;

 if abs(dx) >= abs(dy) then    //horizontal orientation
  begin
   if dx > 0 then stepX := 1 else stepX := -1;
   stepY := dy/abs(dx);
   steps := abs(dx);
  end
  else
   begin
    if dy > 0 then stepY := 1 else stepY := -1;
    stepX := dx/abs(dy);
    steps := abs(dy);
   end;
 pX := x1;   //previous value
 pY := y1;   //..
 for n := 1 to steps do
  begin
   newX := x1 + round(n*stepX);
   newY := y1 + round(n*stepY);
   if newX > pX then code := 1 else code := 0;
   if newX < pX then code := 2;
   if newY > pY then code := code or 4
    else if newY < pY then code := code or 8;
   dir := code2dir[code];
   procmove(dir);
   pX := newX;
   pY := newY;
  end;
end;

procedure paintScanChar(ch : char);
//in scanmap
begin
 with scanmap do with Canvas do
  begin
   font.Name := 'roman';
   brush.Style := bsSolid;
   brush.Color := $000000;
   fillrect(rect(0,0,width,height));  //black background
   brush.color := $ffffff;            //white char background
   font.Color := $0000ff;             //red character
   TextOut(0,0,ch);
  end;
end;

function scanChar(var sx,sy : shortInt) : boolean;
//find next dot of character
//exit false if none
var dw : longword;
    xf,xxf : boolean;
begin
 result := false;
 xf := false;
 xxf := false;
 repeat
  dw := scanmap.canvas.pixels[sx,sy];
  case dw of
   $0000ff : begin
              result := true;
              scanmap.canvas.pixels[sx,sy] := $ffffff;  //clear pixel
              exit;
             end;
   $ffffff : begin
              xf := false;
              inc(sx);
             end;
   $000000 : begin
              xxf := xf;
              xf:= true;
              sx := 0;
              inc(sy);
             end;
  end;
 until result or xxf;
end;

function subScanChar(var x,y : shortInt; var d : byte) : boolean;
//find adjacent character dots, supply direction code
type TSP = record
            dx : shortInt; //offset
            dy : shortInt;
            c  : byte;     //direction code
           end;
const SP : array[0..7] of TSP = ((dx: 0;dy: 1;c:6),(dx: 1;dy: 0;c:0),
                                 (dx: 1;dy: 1;c:7),(dx:-1;dy: 1;c:5),
                                 (dx:-1;dy: 0;c:4),(dx:-1;dy:-1;c:3),
                                 (dx: 0;dy:-1;c:2),(dx: 1;dy:-1;c:1));
var i : byte;
begin
 i := 0;
 result := false;
 repeat
  with SP[i] do
   if scanmap.canvas.pixels[x+dx,y+dy] = $0000ff then
    begin
     result := true;
     x := x + SP[i].dx;
     y := y + SP[i].dy;
     d := c;
     scanmap.Canvas.Pixels[x,y] := $ffffff;   //clear pixel
    end
    else inc(i);
 until result or (i=8);
end;

procedure drawDemoChar(chx,chy : smallInt);
var dir : byte;
    x,y : shortInt;          //subscan coords in scanmap
    scanX,scanY : shortInt;  //scan coords in scanmap
begin
 scanX := 0;
 scanY := 0;
 while scanChar(scanX,scanY) do
  begin
   movetoXY(chx+scanX,chy+scanY);
   setpen(true);
   x := scanX;
   y := scanY;
   while subscanChar(x,y,dir) do procmove(dir);
   setpen(false);
  end;
end;

procedure painttextline(n : byte);
var s : string;
    charX,charY : smallInt;
    charH,charW : byte;
    i : byte;
begin
  CharX := demotext[n].x;
  CharY := demotext[n].y;
  charH := demotext[n].height;
  scanmap.Canvas.Font.Height := charH;
  s := demotext[n].s;
  i := 0;
  while GoDemoFlag and (i < length(s)) do
   begin
    inc(i);
    charW := scanmap.Canvas.TextWidth(s[i]);
    setPen(false);
    movetoXY(charX,charY);
    paintScanChar(s[i]);       //in bitmap for scanning
    drawDemoChar(charX,charY);
    charX := charX + charW;
   end;//line
end;

procedure startdemo(nr : byte);
//n : demo nr.
var x,y : single;
    n : word;
    dt : single;
begin
 setpen(false);
 GoDemoFlag := true;
 case nr of
  1 : begin
       dt := 2*pi/500;
       n := 0;
       while GoDemoFlag and (n < 500) do
        begin
         x := 150*(sin(8*n*dt)+sin(n*dt));
         y := 150*(cos(8*n*dt)+cos(n*dt));
         if n = 0 then begin
                        movetoXY(round(x),round(y));
                        setpen(true);
                       end
          else movetoXY(round(x),round(y));
         inc(n);
        end;
      end;
  2 : begin
       dt := 2*pi/500;
       n := 0;
       while GoDemoFlag and (n < 500) do
        begin
         x := 500*sin(9*n*dt)*cos(9*n*dt)*sin(7*n*dt);
         y := 275*sin(9*n*dt)*cos(7*n*dt);
         if n =0 then begin
                       movetoXY(round(x),round(y));
                       setpen(true);
                      end
          else movetoXY(round(x),round(y));
         inc(n);
        end;
      end;
  3 : begin
       n := 0;
       while (n < High(demotext)) and GoDemoFlag do
        begin
         inc(n);
         painttextline(n);
        end;
      end;
 end;//case
 setpen(false);
end;

procedure createscanmap;
begin
 scanmap := Tbitmap.create;
 with scanmap do
  begin
   width := 60;
   height := 60;
   pixelformat := pf32bit;
  end;
end;

procedure destroyscanmap;
begin
 scanmap.free;
end;

end.
